--******************************************************************************************************************************
--
-- Description: Model 372 product feature description.
--				Note: This package is common to all SigFPGA products, but tailored to the Model 372.
--					
-- Rev History: 11-28-2018:	Added MAX_TAP_VALUE, DMA_ENGINE_COUNT and ADC_CLK_INPUTS.
-- 				08-04-2016:	Added QDR_SPREAD, TX_FRAME_SHARE, TX_ENABLE_SHARE, TX_CLOCK_SHARE and RX_FRAME_SHARE.
-- 				02-18-2016:	Inital release.
--
--******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

package model_specific_package is

	-- Function that will prevent a negative index on designs with no RX channels.
	function zero_to_one (ch_cnt : natural) return natural;
	
	-- Assign default values specific to this model number.
	constant MODEL			: std_logic_vector(11 downto 0) := x"372";		-- Model number.
	constant REVISION		: std_logic_vector(31 downto 0) := x"11282018";	-- FPGA design revision.
	constant GPIO_COUNT		: natural := 6;  		-- Number of GPIO pins.
	constant USER_COUNT		: natural := 62;  		-- Number of user pins.
	constant PCI_LANES		: natural := 8;  		-- Number of PCIe electrical lanes.
	constant AXI_WIDTH		: natural := 128;  		-- Width of AXI bus connecting the EZDMA2 core to the Xilinx endpoint core.
	constant QDR_ADDR_WIDTH	: natural := 0;  		-- Width of QDR II+ SRAM address.
	constant QDR_SPREAD		: boolean := true;		-- Enable or disable spread spectrum clocking on the QDR II+ SRAM.
	constant BPI_ADD		: natural := 23;		-- Size of BPI configuration flash address bus.
	constant MAX_TAP_VALUE	: natural := 31;		-- Maximum IDELAY tap value.
	
	-- Define the transmitter channel characteristics.
	constant TX_CHANNEL_COUNT 	: natural := 2;		-- Two TX channels on this model.
	constant TX_ITEM_SIZE 		: natural := 16;	-- 16-bit DAC.
	constant TX_DATA_SHARE		: natural := 2;		-- Number of channels sharing a data port.
	constant TX_FRAME_SHARE		: natural := 2;		-- Number of channels sharing a frame sync port.
	constant TX_ENABLE_SHARE	: natural := 2;		-- Number of channels sharing an enable.
	constant TX_CLOCK_SHARE		: natural := 2;		-- Number of channels sharing a clock.
	constant TX_ITEM_STUFF 		: natural := 1;		-- Must be greater than zero.
	constant TX_CLK_SELECT 		: natural := 0;		-- Route sample clock to the TX channel.

	-- Define the receiver channel characteristics.
	constant RX_CHANNEL_COUNT 	: natural := 2;		-- Two RX channels on this model.
	constant RX_ITEM_SIZE 		: natural := 16;	-- 16-bit ADC
	constant RX_ITEM_PER_CLK	: natural := 1;		-- Number of items generated per clock edge.
	constant RX_ITEM_STUFF 		: natural := 1;		-- Number of items stuffed into the ADC output bus.
	constant RX_INTERLEAVE 		: natural := 1;		-- Bits are not interleaved across DDR clock edges.
	constant RX_DATA_SHARE		: natural := 2;		-- Number of channels sharing a data port.
	constant RX_FRAME_SHARE		: natural := 100;	-- Number of channels sharing a frame sync (natural >= 1).
	constant RX_CHAN_PER_OVR	: natural := 1;		-- Number of channels assigned to each over-range bit.
	constant RX_ITEM_PER_OVR	: natural := 1;		-- Number of items assigned to each over-range bit.
	constant RX_CLK_SELECT 		: natural := 0;		-- Route sample clock to the RX channel.
	
	-- Set the number of active DMA channels.
	constant DMA_CHANNEL_COUNT 	: natural := TX_CHANNEL_COUNT+RX_CHANNEL_COUNT;		-- Match the number of RX/TX channels.
	constant DMA_ENGINE_COUNT 	: natural := TX_CHANNEL_COUNT+RX_CHANNEL_COUNT;		-- Number of DMA engines to share among channels (<= 7).
	
	-- The following values are hardware specific to the model number ADC interface.
	type ADC_IO_DELAY is array (0 to ((RX_CHANNEL_COUNT/RX_DATA_SHARE)*RX_ITEM_SIZE/RX_INTERLEAVE)-1) of natural;
	-- Set the ADC data and over-range IOB initial delay values.
	-- Note:  The position of each initialization value will match the array index, so the zero index is on the left.
	constant ADC_DATA_INIT		: ADC_IO_DELAY := (0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0);
	constant ADC_OVR_INIT		: natural := 0;
	-- Do not offset the clock forwarded by the ADC.
	constant ADC_CLK_INIT		: natural := 0;
	-- Command the number of ADC clock forward sources to evaluate.
	-- Note: There is one source supplied by each dual ADC chip.
	constant ADC_CLK_SOURCES	: natural := 1;
	constant ADC_CLK_INPUTS		: natural := 1;
	-- Command the clock lock function to search for the rising and falling edge of the forwarded ADC clock.
	constant ADC_CLK_EDGE		: std_logic_vector(1 downto 0) := "11";
	-- Set the maximum IDELAY/ODELAY chain delay count.
	constant ADC_MAX_SEARCH		: natural := 31;
	-- Return the clock lock process to idle if a solution is not found after the search.
	constant ADC_RESET_IF_NOSOL	: boolean := true;

	-- The following values are hardware specific to the model number DAC interface.
	-- Command the number of DAC clock forward sources to generate.
	-- Note: There is one source supplied by each dual DAC chip.
	constant DAC_CLK_SOURCES	: natural := 1;

	-- This model is not equipped with QDR SRAM.
	constant QDR_PERIOD		: natural := 0;

end model_specific_package;

package body model_specific_package is

	-- Convert zero to one for the channel count index.
	function zero_to_one (ch_cnt : natural) return natural is
	begin
		if ch_cnt = 0 then
			return 1;
		else
			return ch_cnt;
		end if;
	end zero_to_one;

end model_specific_package;




