--*******************************************************************************************************************************
--
-- Description: Model 377 top level entity.
--				Note: This entity is common to all SigFPGA products, but tailored to the Model 377.
--					
-- Rev History: 01-10-2018:	Removed port tx_clk_in from the transceiver model.
--				08-04-2016:	Added ports rx_frame_in, tx_frame_out and tx_enable_out to the transceiver model.
-- 				02-18-2016:	Changed type of rx_ovr_in_iob signal to accommodate a new ADC.
-- 				08-06-2015:	Defined DIFF_TERM generic for all differential LVDS inputs.
-- 				07-14-2015:	Changed pinout to the transceiver entity to accommodate removal of the QDR MIG core.
-- 				12-08-2014:	Moved GPIO and USER control bits (direction/tristate) to BAR0 registers.
-- 				10-13-2014:	Removed pin contraints for EMCCLK to work around a bug in Vivado 2014.2 and 2014.3.
--				08-20-2014:	Added fan_clear/fan_status and support for bidirectional SPI bus (SDIO), changed polarity of GPIO and USER direction.
-- 				09-04-2013:	Initial release.
--                                                  
--*******************************************************************************************************************************
                                             
library ieee,unisim;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;
use unisim.VCOMPONENTS.all;
use work.model_specific_package.all;
use work.transceiver_package.all;

entity SigFPGA_Top is
port (
	-- PCI Express bus signals produced by the Xilinx Endpoint Block Plus for PCI Express core generator.
	sys_reset_n		: in std_logic;						-- Refer to the Xilinx LogiCORE IP Endpoint for PCI Express User Guide.
	
	pci_sys_clk_p	: in std_logic;						-- Refer to the Xilinx LogiCORE IP Endpoint for PCI Express User Guide.
	pci_sys_clk_n	: in std_logic;						-- Differential pair (see above).
	
	pci_exp_rxp		: in std_logic_vector(PCI_LANES-1 downto 0);	-- Refer to the Xilinx LogiCORE IP Endpoint for PCI Express User Guide.
	pci_exp_rxn		: in std_logic_vector(PCI_LANES-1 downto 0);	-- Differential pair (see above).
	
	pci_exp_txp		: out std_logic_vector(PCI_LANES-1 downto 0);	-- Refer to the Xilinx LogiCORE IP Endpoint for PCI Express User Guide.
	pci_exp_txn		: out std_logic_vector(PCI_LANES-1 downto 0);	-- Differential pair (see above).
	
	-- 200 MHz clock from an external oscillator.
	clk200_p		: in std_logic;						-- 200 MHz clock for IDELAY reference.
	clk200_n		: in std_logic;						-- Differential pair (see above).
	
	-- Sample clock from an external frequency synthesizer.
	sample_clk_p  	: in std_logic;						-- Sample clock distributed to the ADC/DAC.
	sample_clk_n  	: in std_logic;						-- Differential pair (see above).
	
	-- RX data, over-range (ovr) and forwarded clock signals.
	rx1_data_in_p	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- RX channel #1 data bus.
	rx1_data_in_n	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- Differential pair (see above).
	   
	rx1_ovr_in		: in std_logic;						-- RX channel #1 over-range indicator.
	                                                                                                   
	rx2_data_in_p	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- RX channel #2 data bus.
	rx2_data_in_n	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- Differential pair (see above).
	
	rx2_ovr_in		: in std_logic;						-- RX channel #2 over-range indicator.
	
	rx1to2_clk_in_p	: in std_logic;						-- RX channel #1/#2 forwarded clock.
	rx1to2_clk_in_n	: in std_logic;						-- Differential pair (see above).
	                
	-- RX channel #2 shares the RX channel #1 forwarded clock.

	rx3_data_in_p	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- RX channel #3 data bus.
	rx3_data_in_n	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- Differential pair (see above).
	
	rx3_ovr_in		: in std_logic;						-- RX channel #3 over-range indicator.
	
	rx4_data_in_p	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- RX channel #4 data bus.
	rx4_data_in_n	: in std_logic_vector((RX_ITEM_SIZE/RX_INTERLEAVE)-1 downto 0);	-- Differential pair (see above).
	
	rx4_ovr_in		: in std_logic;						-- RX channel #4 over-range indicator.
	
	rx3to4_clk_in_p	: in std_logic;						-- RX channel #3/#4 forwarded clock.
	rx3to4_clk_in_n	: in std_logic;						-- Differential pair (see above).

	-- Note: RX channel #4 shares the RX channel #3 forwarded clock.

	-- Inter-integrated circuit (I2C) bus signals.
	i2c_scl			: out std_logic;					-- I2C programming clock.
	i2c_sda			: inout std_logic;					-- I2C bidirectional data to peripheral chips.
	
	-- Serial peripheral interface (SPI) bus signals.
	spi_clk			: out std_logic;					-- SPI programming clock.
	spi_sdi			: out std_logic;					-- SPI data output to peripheral chips.
	spi_sdo			: in std_logic;						-- SPI data input from peripheral chips.
	spi_csb_cmd     : out std_logic_vector(2 downto 0); -- SPI chip select command interface (Bit 0: Clock, Bit 1: Data, Bit 2: Enable).
	
	-- Frequency synthesizer command interface signals, applies only to the programmable synthesizer option.
	synth_cmd		: out std_logic_vector(2 downto 0);	-- Dedicated serial command bus to the frequency synthesizer (Bit 0: Clock, Bit 1: Data, Bit 2: Enable). 
	
	-- LED control signals.
	led_on			: out std_logic_vector(1 downto 0);	-- Active high control to switch on LEDs.
	
	-- Hardware trigger inputs.
	coax_trig		: in std_logic;						-- Trigger input on coaxial cable.
	gpio_trig		: in std_logic;						-- Trigger input on GPIO cable.
	
	-- Alert notification from the power monitor and command/status port expander.
	alert_n			: in  std_logic;					-- Active low status alert.
	
	-- Control and status of the fan attached to the active heat sink on native PCIe products.
	-- Note: This function is only available on native PCIe products, not XMC attached to a PCIe carrier.
	fan_clear		: out std_logic;					-- Active high reset to the fan monitor on native PCIe cards.
	fan_status		: in std_logic;						-- Sticky status bit indicating whether fan is active (1) or has failed (0).
	
	-- Uncommitted bidirectional IO pins (GPIO connector and user connector).
	gpio			: inout std_logic_vector(GPIO_COUNT downto 1);	-- GPIO data pins.
	gpio_tristate	: out std_logic;								-- Tristate all GPIO voltage translators.
	user			: inout std_logic_vector(USER_COUNT downto 1);	-- User data pins.
	user_tristate	: out std_logic;								-- Tristate all user voltage translators.
	gpio_user_cmd	: out std_logic_vector(2 downto 0);				-- GPIO/user voltage translator command interface (Bit 0: Clock, Bit 1: Data, Bit 2: Load).
	
	-- Configuration flash byte peripheral interface (BPI).
	bpi_a			: out std_logic_vector(BPI_ADD-1 downto 0);	-- Address bus.
	bpi_rs			: out std_logic_vector(1 downto 0);			-- Revision select.
	bpi_d			: inout std_logic_vector(15 downto 0);		-- Data input (read) bus.
	bpi_adv_b		: out std_logic; 							-- Active low address valid.
	bpi_fcs_b		: out std_logic; 							-- Active low chip enable.
	bpi_fwe_b		: out std_logic; 							-- Active low write enable.
	bpi_foe_b		: out std_logic; 							-- Active low output enable.
	bpi_vpp			: out std_logic 							-- Active high enable voltage for program and erase operations.
);                                                                                            
end SigFPGA_Top;

architecture structural of SigFPGA_Top is

	signal sys_rst_n		: std_logic;					-- Internal system reset.
	signal pci_sys_clk		: std_logic;					-- Internal single ended version of PCI Express reference clock.
	signal clk200_iob		: std_logic;					-- Internal single ended version of differential 200 MHz clock.
	signal sample_clk_iob	: std_logic;					-- Internal single ended version of differential sample clock. 
	signal rx_data_in_iob	: RX_BUS_IN;					-- Internal RX data bus.
	signal rx_ovr_in_iob	: OVR_BIT_IN;					-- Internal RX over-range indicator.
	signal rx_clk_in_iob	: RX_CLOCK_BIT;					-- Internal RX forwarded clock.
	signal qdriip_cq		: DUAL_1;						-- Echo clock.
	signal qdriip_q			: DUAL_18;						-- Synchronous data outputs from the SRAM.
	signal qdriip_qvld		: DUAL_1;						-- Echo clock.
	signal led_on_iob		: std_logic_vector(1 downto 0);	-- Internal active high control to switch on LEDs.
    signal i2c_scl_iob		: std_logic;					-- Internal I2C programming clock.
    signal i2c_sda_in		: std_logic;					-- Internal I2C data input from peripheral chips.
    signal i2c_sda_out		: std_logic;					-- Internal I2C data output to peripheral chips.
    signal i2c_direction	: std_logic;					-- Select I2C direction as input (1) or output (0).
    signal spi_clk_iob		: std_logic;					-- Internal SPI programming clock.
    signal spi_sdi_iob		: std_logic;					-- Internal SPI data output to peripheral chips.
    signal spi_sdo_iob		: std_logic;					-- Internal SPI data input from peripheral chips.
    signal spi_direction	: std_logic;					-- Enable (0) or tristate (1) SDI for bidirectional operation.
    signal spi_csb_cmd_iob	: std_logic_vector(2 downto 0);	-- Internal SPI chip select command interface.
    signal synth_cmd_iob	: std_logic_vector(2 downto 0);	-- Internal frequency synthesizer command interface. 
    signal coax_trig_iob	: std_logic;					-- Trigger input on coaxial cable.
    signal gpio_trig_iob	: std_logic;					-- Trigger input on GPIO cable.
    signal alert_n_iob		: std_logic;					-- Active low alert.
    signal fan_clear_iob	: std_logic;					-- Active high clear to the FPGA fan monitor on native PCIe.
    signal fan_status_iob	: std_logic;					-- Sticky status bit indicating whether fan is active (1) or has failed (0).
    signal gpio_in			: std_logic_vector(GPIO_COUNT downto 1);	-- Internal GPIO inputs. 
    signal gpio_out			: std_logic_vector(GPIO_COUNT downto 1);	-- Internal GPIO outputs. 
    signal gpio_direction	: std_logic_vector(GPIO_COUNT downto 1);	-- Set the direction of GPIO pins to outputs (0) or inputs (1). 
    signal gpio_tristate_iob: std_logic;					-- Tristate all GPIO voltage translators. 
    signal gpio_control		: std_logic_vector(GPIO_COUNT downto 1);	-- Combination of direction and tristate settings. 
    signal user_in			: std_logic_vector(USER_COUNT downto 1);	-- Internal user bus inputs. 
    signal user_out			: std_logic_vector(USER_COUNT downto 1);	-- Internal user bus outputs. 
    signal user_direction	: std_logic_vector(USER_COUNT downto 1);	-- Direction of user pin. 
    signal user_tristate_iob: std_logic;					-- Tristate all user voltage translators. 
    signal user_control		: std_logic_vector(USER_COUNT downto 1);	-- Combination of direction and tristate settings. 
    signal gpio_user_cmd_iob: std_logic_vector(2 downto 0);	-- Internal GPIO/user voltage translator command interface.
	signal bpi_a_iob		: std_logic_vector(BPI_ADD-1 downto 0);-- Address bus.
	signal bpi_rs_iob		: std_logic_vector(1 downto 0);	-- Revision select.
	signal bpi_rs_disable	: std_logic; 								-- Revision select disable.
	signal bpi_adv_b_iob	: std_logic; 					-- Active low address valid.
	signal bpi_fcs_b_iob	: std_logic; 					-- Active low chip enable.
	signal bpi_fwe_b_iob	: std_logic; 					-- Active low write enable.
	signal bpi_foe_b_iob	: std_logic; 					-- Active low output enable.
	signal bpi_vpp_iob		: std_logic; 					-- Active high enable voltage for program and erase operations.
	signal bpi_d_in			: std_logic_vector(15 downto 0);-- Data input (read) bus.
	signal bpi_d_out		: std_logic_vector(15 downto 0);-- Data output (write) bus.
	signal bpi_d_tristate	: std_logic;					-- Active high data bus tristate.

begin

    XCVR : entity work.transceiver
	port map (
		sys_rst_n			=> sys_rst_n,
		pci_sys_clk			=> pci_sys_clk,
		pci_exp_rxp			=> pci_exp_rxp,
		pci_exp_rxn			=> pci_exp_rxn,
		pci_exp_txp			=> pci_exp_txp,
		pci_exp_txn			=> pci_exp_txn,
		clk200_in			=> clk200_iob,
		sample_clk_in		=> sample_clk_iob,
		rx_data_in 			=> rx_data_in_iob,
		rx_frame_in 		=> (others => '0'),
		rx_ovr_in 			=> rx_ovr_in_iob,
		rx_clk_in 			=> rx_clk_in_iob,
		tx_data_out 		=> open,
		tx_frame_out 		=> open,
		tx_enable_out 		=> open,
		tx_clk_out			=> open,
		qdriip_sys_clk		=> '0',
		qdriip_cq			=> qdriip_cq,
		qdriip_q			=> qdriip_q,
		qdriip_qvld			=> qdriip_qvld,
		qdriip_k			=> open,
		qdriip_d			=> open,
		qdriip_sa			=> open,
		qdriip_w_n			=> open,
		qdriip_r_n			=> open,
		qdriip_bw_n			=> open,
		qdriip_dll_off_n	=> open,
		i2c_scl 			=> i2c_scl_iob,
		i2c_sda_in 			=> i2c_sda_in,
		i2c_sda_out 		=> i2c_sda_out,
		i2c_direction 		=> i2c_direction,
		spi_clk 			=> spi_clk_iob,
		spi_sdi 			=> spi_sdi_iob,
		spi_sdo 			=> spi_sdo_iob,
		spi_direction		=> spi_direction,
		spi_csb_cmd 		=> spi_csb_cmd_iob,
		synth_cmd			=> synth_cmd_iob,
		led_on 				=> led_on_iob,
		coax_trig			=> coax_trig_iob,
		gpio_trig			=> gpio_trig_iob,
		alert_n				=> alert_n_iob,
		fan_clear			=> fan_clear_iob,
		fan_status			=> fan_status_iob,
		gpio_in				=> gpio_in,
		gpio_out			=> gpio_out,
		gpio_direction		=> gpio_direction,
		gpio_tristate		=> gpio_tristate_iob,
		user_in				=> user_in,
		user_out			=> user_out,
		user_direction		=> user_direction,
		user_tristate		=> user_tristate_iob,
		gpio_user_cmd		=> gpio_user_cmd_iob,
		bpi_a				=> bpi_a_iob,
		bpi_rs				=> bpi_rs_iob,
		bpi_rs_disable		=> bpi_rs_disable,
		bpi_d_in			=> bpi_d_in,
		bpi_d_out			=> bpi_d_out,
	    bpi_d_tristate		=> bpi_d_tristate, 
		bpi_adv_b			=> bpi_adv_b_iob,
		bpi_fcs_b			=> bpi_fcs_b_iob,
		bpi_fwe_b			=> bpi_fwe_b_iob,
		bpi_foe_b			=> bpi_foe_b_iob,
		bpi_vpp				=> bpi_vpp_iob
	);

    -- Assign the PCI Express differential system clock to a global clock tree.                    
    REF_CLK_IBUF : IBUFDS_GTE2 port map (I => pci_sys_clk_p, IB => pci_sys_clk_n, O => pci_sys_clk, CEB => '0', ODIV2 => open);

	--	Map the differential 200 MHz external clock input to a global clock buffer.	
	CLK200_IBUF : IBUFGDS
		generic map (DIFF_TERM  => FALSE, IOSTANDARD => "LVDS_25")
		port map (I => clk200_p, IB => clk200_n, O => clk200_iob);

    --	Map the LVDS sample clock input pair to a global clock buffer.
    SAMPLE_CLK_IBUF : IBUFGDS
		generic map (DIFF_TERM  => FALSE, IOSTANDARD => "LVDS_25")
    	port map (I => sample_clk_p, IB=> sample_clk_n, O=> sample_clk_iob);

	-- Map the system reset to an internal signal.
	SYS_RESET_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => sys_rst_n, I => sys_reset_n);
		
	-- Map the LVDS RX data pin pairs to an internal data bus.
	RX_DATA_BUF : for i in 0 to (RX_ITEM_PER_CLK*RX_ITEM_SIZE/RX_INTERLEAVE)-1 generate
		RX1_DATA_IBUF : IBUFDS  -- Channel 1
			generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
			port map (I => rx1_data_in_p(i), IB => rx1_data_in_n(i), O => rx_data_in_iob(1)(i));
		RX2_DATA_IBUF : IBUFDS  -- Channel 2
			generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
			port map (I => rx2_data_in_p(i), IB => rx2_data_in_n(i), O => rx_data_in_iob(2)(i));
		RX3_DATA_IBUF : IBUFDS  -- Channel 3
			generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
			port map (I => rx3_data_in_p(i), IB => rx3_data_in_n(i), O => rx_data_in_iob(3)(i));
		RX4_DATA_IBUF : IBUFDS  -- Channel 4
			generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
			port map (I => rx4_data_in_p(i), IB => rx4_data_in_n(i), O => rx_data_in_iob(4)(i));
	end generate;
	
	--	Map the RX over-range pins to internal signals.
	RX1_OVR_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => rx_ovr_in_iob(1)(0), I => rx1_ovr_in);
	RX2_OVR_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => rx_ovr_in_iob(2)(0), I => rx2_ovr_in);
	RX3_OVR_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => rx_ovr_in_iob(3)(0), I => rx3_ovr_in);
	RX4_OVR_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => rx_ovr_in_iob(4)(0), I => rx4_ovr_in);
		
	--	Map the LVDS RX forwarded clock pin pairs to internal signals.
	RX1TO2_CLK_IBUF : IBUFDS  -- Channel 1 & 2
		generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
	 	port map (I  => rx1to2_clk_in_p, IB => rx1to2_clk_in_n, O => rx_clk_in_iob(1));
	RX3TO4_CLK_IBUF : IBUFDS  -- Channel 3 & 4
		generic map (DIFF_TERM  => TRUE, IOSTANDARD => "LVDS_25")
	 	port map (I  => rx3to4_clk_in_p, IB => rx3to4_clk_in_n, O => rx_clk_in_iob(2));
	 	
	-- Assign the I2C bus pins to I/O buffers.
	I2C_SCL_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => i2c_scl_iob, O => i2c_scl);
	I2C_SDA_OBUF : IOBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => i2c_sda_in, IO => i2c_sda, I => i2c_sda_out, T => i2c_direction);
		
	-- Assign the SPI bus pins to I/O buffers.
	SPI_CLK_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => spi_clk_iob, O => spi_clk);
	SPI_SDI_OBUF : OBUFT
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => spi_sdi_iob, O => spi_sdi, T => spi_direction);
	SPI_SDO_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => spi_sdo_iob, I => spi_sdo);
	SPI_CSB_CMD_BUF : for i in 0 to 2 generate
		SPI_CSB_CMD_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => spi_csb_cmd_iob(i), O => spi_csb_cmd(i));
	end generate;
		
	-- Assign the frequency synthesizer command bus pins to I/O buffers.
	SYNTH_BUF : for i in 0 to 2 generate
		SYNTH_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => synth_cmd_iob(i), O => synth_cmd(i));
	end generate;
		
	--	Assign the LED controls to output buffers.
	LED_BUF : for i in 0 to 1 generate
		LED_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => led_on_iob(i), O => led_on(i));
	end generate;
	 	
	-- Map the trigger pins to internal signals.
	COAX_TRIG_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => coax_trig_iob, I => coax_trig);
	GPIO_TRIG_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => gpio_trig_iob, I => gpio_trig);
	
	-- Map the alert pin to an internal signal.
	ALERT_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (O => alert_n_iob, I => alert_n);
		
	-- Map the fan clear to an internal signal.
	FAN_CLEAR_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS25")
		port map (I => fan_clear_iob, O => fan_clear);
	-- Map the fan clear to an internal signal.
	FAN_STATUS_IBUF : IBUF
		generic map (IOSTANDARD => "LVCMOS25")
		port map (O => fan_status_iob, I => fan_status);
		
	-- Map the GPIO pins to an internal bus.
	GPIO_BUF : for i in 1 to GPIO_COUNT generate
		-- Tristate the pin if it is defined as an input or the global tristate bit is enabled.
		gpio_control(i) <= not(gpio_direction(i)) or gpio_tristate_iob;
		GPIO_IOBUF : IOBUF
			generic map (IOSTANDARD => "LVCMOS18", DRIVE => 4)
			port map (O => gpio_in(i), IO => gpio(i), I => gpio_out(i), T => gpio_control(i));
	end generate;
	-- Tristate the voltage translators connected to the GPIO pins if the global tristate bit is enabled.
	GPIO_TRISTATE_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => gpio_tristate_iob, O => gpio_tristate);

	-- Map the USER pins to an internal bus.
	USER_BUF : for i in 1 to USER_COUNT generate
		-- Tristate the pin if it is defined as an input or the global tristate bit is enabled.
		user_control(i) <= not(user_direction(i)) or user_tristate_iob;
		-- Pin numbers 1 through 50 are powered by either 2.5V (default) or 3.3V (build option).
		USER_50 : if i <= 50 generate
			DIRECT_IOBUF : IOBUF
				generic map (IOSTANDARD => "LVCMOS25")
				port map (O => user_in(i), IO => user(i), I => user_out(i), T => user_control(i));
		end generate;
		-- Pin numbers greater than 50 are powered by 1.8V and connect through a voltage translator.
		USER_GT50 : if i > 50 generate
			XLATOR_IOBUF : IOBUF
				generic map (IOSTANDARD => "LVCMOS18", DRIVE => 4)
				port map (O => user_in(i), IO => user(i), I => user_out(i), T => user_control(i));
		end generate;
	end generate;
	-- Tristate the voltage translators connected to the USER pins if the global tristate bit is enabled.
	USER_TRISTATE_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => user_tristate_iob, O => user_tristate);

	-- Map the GPIO/user voltage translator command interface pins to an internal bus.
	GPIO_USER_CMD_BUF : for i in 0 to 2 generate
		GPIO_USER_CMD_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => gpio_user_cmd_iob(i), O => gpio_user_cmd(i));
	end generate;
	
	-- Map the BPI configuration flash pins to an internal bus.
	BPI_ADDR : for i in 0 to BPI_ADD-1 generate
		ADD_OBUF : OBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (I => bpi_a_iob(i), O => bpi_a(i));
	end generate;
	BPI_REV : for i in 0 to 1 generate
		RS_OBUFT : OBUFT
			generic map (IOSTANDARD => "LVCMOS18")
			port map ( O => bpi_rs(i), I => bpi_rs_iob(i), T => bpi_rs_disable);
	end generate;
	BPI_DATA : for i in 0 to 15 generate
		DATA_IOBUF : IOBUF
			generic map (IOSTANDARD => "LVCMOS18")
			port map (O => bpi_d_in(i), IO => bpi_d(i), I => bpi_d_out(i), T => bpi_d_tristate);
	end generate;
	ADV_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => bpi_adv_b_iob, O => bpi_adv_b);
	FCS_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => bpi_fcs_b_iob, O => bpi_fcs_b);
	FWE_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => bpi_fwe_b_iob, O => bpi_fwe_b);
	FOE_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => bpi_foe_b_iob, O => bpi_foe_b);
	VPP_OBUF : OBUF
		generic map (IOSTANDARD => "LVCMOS18")
		port map (I => bpi_vpp_iob, O => bpi_vpp);

	-- Tie off unused QDR II SRAM inputs to transceiver.
	qdriip_cq(0) <= '0';
	qdriip_q(0) <= (others => '0');
	qdriip_qvld(0) <= '0';
	qdriip_cq(1) <= '0';
	qdriip_q(1) <= (others => '0');
	qdriip_qvld(1) <= '0';
	
end structural;