--******************************************************************************************************************************
--
-- Description: BPI configuration flash programmer that can be commanded from software.
--					
-- Rev History: 11-28-2018:	Added a synchronous reset and converted to an AXI bus structure.
-- 				05-12-2015:	Added revision select command and substituted IP generated MMCM with macro equivalent.
--				04-17-2015:	Added functionality to program flash through the PCI interface.
--				01-29-2014:	Initial release.
--
--******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

Library UNISIM;
use UNISIM.vcomponents.all;
library UNIMACRO;
use unimacro.Vcomponents.all;

use work.model_specific_package.all;

entity bpi_flash is
	port (
    -- Reset and clock.                                                                                      
	srst    			: in std_logic;								-- Active high synchronous reset in the system clock domain.
	clk 				: in std_logic;								-- System clock.
	clk200 				: in std_logic;								-- 200 MHz clock from an external oscillator.

	-- Connections to command/status registers in the BAR0 module.
	bpi_enable			: in std_logic; 							-- Active high BPI programming enable.
	bpi_cmd_tvalid		: in std_logic; 							-- Logic one writes the next BPI command.
	bpi_cmd_tready		: out std_logic; 							-- Logic one indicates the BPI command FIFO can accept another burst.
	bpi_cmd_tdata		: in std_logic_vector(31 downto 0);			-- Command bus that carries instructions and data to program the BPI flash.
	bpi_reply_tvalid	: out std_logic; 							-- Logic indicates that a BPI reply is available.
	bpi_reply_tready	: in std_logic; 							-- Logic one pops the BPI reply FIFO to the next value.
	bpi_reply_tdata		: out std_logic_vector(31 downto 0); 		-- Reply bus that returns data and errors from the BPI flash.
	clk20_locked		: out std_logic; 							-- Logic one indicates MMCM locked.
	
	--	The following signals connect to external devices through FPGA pins.
	bpi_a				: out std_logic_vector(BPI_ADD-1 downto 0);	-- Address bus.
	bpi_rs				: out std_logic_vector(1 downto 0);			-- Revision address.
	bpi_rs_disable		: out std_logic; 							-- Revision select disable.
	bpi_d_in			: in std_logic_vector(15 downto 0);			-- Data input (read) bus.
	bpi_d_out			: out std_logic_vector(15 downto 0);		-- Data output (write) bus.
	bpi_d_tristate		: out std_logic;							-- Active high data bus tristate.
	bpi_adv_b			: out std_logic; 							-- Active low address valid.
	bpi_fcs_b			: out std_logic; 							-- Active low chip enable.
	bpi_fwe_b			: out std_logic; 							-- Active low write enable.
	bpi_foe_b			: out std_logic; 							-- Active low output enable.
	bpi_vpp				: out std_logic 							-- Active high enable voltage for program and erase operations.
	);
end bpi_flash;

architecture structural of bpi_flash is

	type PROGRAM_STATE is (idle, revision, address, write, read1, read2, read3, pop);	--	Type definitions for the state machine.
	signal bpi_cmd			: std_logic_vector(1 downto 0);		-- Status bits from the BPI command FIFO.
	signal bpi_srst			: std_logic;						-- Active high synchronous reset in the 20 MHz clock domain.
	signal bpi_state 		: PROGRAM_STATE;					-- Current state.
	signal clk20			: std_logic;						-- 20 MHz clock on a global clock bufffer.
	signal clk20fbin		: std_logic;						-- MMCM feedback output on a global clock buffer.
	signal clk20fbout		: std_logic;						-- MMCM feedback output.
	signal clk20out0		: std_logic;						-- MMCM input clock divided by 10.
	signal command_enable	: std_logic_vector(1 downto 0);		-- Active high enable to the BPI command state machine.
	signal command_tvalid	: std_logic;						-- Logic one indicates a valid command on the bus.
	signal command_tready	: std_logic;						-- Command bus read enable.
	signal command_tdata	: std_logic_vector(31 downto 0);	-- Command bus that carries instructions and data to program the flash.
	signal reply			: std_logic_vector(1 downto 0);		-- Status bits from the BPI reply FIFO.
	signal reply_tvalid		: std_logic;						-- Reply bus write enable.
	signal reply_tready		: std_logic;						-- Logic one indicates the BPI reply FIFO can accept another burst.
	signal reply_tdata		: std_logic_vector(31 downto 0);	-- Reply bus that returns data and errors from the BPI flash.
	
begin

--**************************************************** INSTANTIATE CLOCK *******************************************************

    -- Divide the 200 MHz clock by ten to produce a 20 MHz clock for the programmer.
    -- Note: The MMCM automatically locks after power-on.
	CLK200_CLK20 : MMCME2_BASE
	generic map (
		CLKFBOUT_MULT_F => 42.375,
		CLKIN1_PERIOD => 5.0,
		CLKOUT0_DIVIDE_F => 42.375,
		DIVCLK_DIVIDE => 10
	)
	port map (
		CLKFBIN => clk20fbin,
		CLKFBOUT => clk20fbout,
		CLKFBOUTB => open,
		CLKOUT0 => clk20out0,
		CLKOUT0B => open,
		CLKOUT1 => open,
		CLKOUT1B => open,
		CLKOUT2 => open,
		CLKOUT2B => open,
		CLKOUT3 => open,
		CLKOUT3B => open,
		CLKOUT4 => open,
		CLKOUT5 => open,
		CLKOUT6 => open,
		LOCKED => clk20_locked,
		CLKIN1 => clk200,
		PWRDWN => '0',
		RST => '0'
	);
	-- Map the MMCM feedback to a global clock buffer.
	CLKFB_BUFG : BUFG port map (I => clk20fbout, O => clk20fbin);
	-- Map the MMCM divided output to a 20 MHz global clock buffer.
	CLKOUT0_BUFG : BUFG port map (I => clk20out0, O => clk20);	
	
--**************************************************** INSTANTIATE FIFOS *******************************************************

    -- FIFO to buffer incoming commands and data.
    -- Note: Reset is edge-sensitive, so apply whenever the flash is disabled to overcome latency.
   	COMMAND_FIFO : entity work.fifo_flash_512x32
	port map (
		s_aresetn 			=> bpi_enable,
		s_aclk 				=> clk,
		s_axis_tvalid		=> bpi_cmd_tvalid,
		s_axis_tready		=> bpi_cmd(0),
		s_axis_tdata		=> bpi_cmd_tdata,
		axis_prog_full 		=> bpi_cmd(1),
		wr_rst_busy 		=> open,
		m_aclk 				=> clk20,
		m_axis_tvalid		=> command_tvalid,
		m_axis_tready		=> command_tready,
		m_axis_tdata		=> command_tdata,
		rd_rst_busy 		=> open
	);
	bpi_cmd_tready <= bpi_cmd(0) and not(bpi_cmd(1));
	
    -- FIFO to buffer outgoing data and error messages.
    -- Note: Reset is edge-sensitive, so apply whenever the flash is disabled to overcome latency.
   	REPLY_FIFO : entity work.fifo_flash_512x32
	port map (
		s_aresetn 			=> bpi_enable,
		s_aclk 				=> clk20,
		s_axis_tvalid		=> reply_tvalid,
		s_axis_tready		=> reply(0),
		s_axis_tdata		=> reply_tdata,
		axis_prog_full 		=> reply(1),
		wr_rst_busy 		=> open,
		m_aclk 				=> clk,
		m_axis_tvalid		=> bpi_reply_tvalid,
		m_axis_tready		=> bpi_reply_tready,
		m_axis_tdata		=> bpi_reply_tdata,
		rd_rst_busy 		=> open
	);
	reply_tready <= reply(0) and not(reply(1));
	
--************************************************* STATE MACHINE TRANSITIONS **************************************************

	process (clk20)
    begin
        if rising_edge(clk20) then
        
        	-- Reclock the programming enable bit in the 20 MHz clock domain.
        	command_enable(0) <= bpi_enable;
        	command_enable(1) <= command_enable(0);
        
        	if command_enable(1) = '0' then
				bpi_state <= idle;
			
			else
				case bpi_state is
					-- Remain idle until a command is available.
					when idle =>
						-- Monitor the command/data FIFO for a new command.
						-- Note: The enable command (0) is processed by the BAR0 decoder.
						if command_tvalid = '1' then
							case command_tdata(31 downto 28) is
								-- Note Command x"0" is intercepted by the BAR0 decoder.
								when x"1" => bpi_state <= revision;
								when x"2" => bpi_state <= address;
								when x"3" => bpi_state <= write;
								when x"4" => bpi_state <= read1;
								-- Pop any unrecognized commands.
								when others => bpi_state <= pop;
							end case;
						end if;
						
					-- Store the revision select settings.
					when revision =>
						bpi_state <= pop;
						
					-- Store the address settings.
					when address =>
						bpi_state <= pop;
						
					-- Perform a command write operation.
					when write =>
						bpi_state <= pop;
						
					-- Tri-state the FPGA data outputs.
					when read1 =>
						bpi_state <= read2;
					-- Enable the BPI outputs while FPGA data outputs are tri-state.
					when read2 =>
						-- Do not continue if the reply FIFO is not ready.
						if reply_tready = '1' then
							bpi_state <= read3;
						else
							bpi_state <= read2;
						end if;
					-- Store the read data and disable the BPI outputs while FPGA data outputs are tri-state.
					-- Note: Bus will turn-around back to write flash during the next state (pop).
					when read3 =>
						bpi_state <= pop;
						
					-- Pop the command FIFO to receive the next instruction or data.
					when pop =>
						bpi_state <= idle;
						
					when others  =>
						bpi_state <= idle;
				end case;
			end if;
        end if;
    end process;

--*************************************************** STATE MACHINE EVENTS *****************************************************

	-- Create a synchronous reset in the 20 MHz clock domain.
	SYNC_TXRST : entity work.sync_reset
	port map (
		source_clk		=> clk,
		source_rst		=> srst,
		target_clk  	=> clk20,
		target_rst		=> bpi_srst
	);
	
	process (clk20)
    begin
        if rising_edge(clk20) then
        
			if bpi_srst = '1' then
			
				bpi_a <= (others => '0');
				-- The following two resets must remain together for the register to pack into the IOB.
				bpi_d_out <= (others => '0');
				bpi_d_tristate <= '0';
				bpi_rs <= (others => '0');
				bpi_rs_disable <= '1';
				reply_tdata(15 downto 0) <= (others => '0');
				
			else
	
				-- Store the BPI revision select settings when commanded.
				if bpi_state = revision then
					-- Assign the revision select bits.
					bpi_rs <= command_tdata(1 downto 0);
					bpi_rs_disable <= not(command_tdata(2));
				end if;
				
				-- Store the BPI address when commanded.
				if bpi_state = address then
					-- Assign the address bits.
					bpi_a <= command_tdata(22 downto 0);
				end if;
				
				-- Send a write command to the flash.
				if bpi_state = write then
					-- Assign the data bits.
					bpi_d_out <= command_tdata(15 downto 0);
					-- Set the write enable active low.
					bpi_fwe_b <= '0';
				else
					bpi_fwe_b <= '1';
				end if;
						
				-- Tristate the FPGA data bus throughout a flash read transaction.
				if bpi_state = read1 or bpi_state = read2 or bpi_state = read3 then
					bpi_d_tristate <= '1';
				else
					bpi_d_tristate <= '0';
				end if;
						
				-- Enable the BPI data bus during a flash read transaction.
				if bpi_state = read2 then
					bpi_foe_b <= '0';
				else
					bpi_foe_b <= '1';
				end if;
				
				-- Return the results from a read operation.
				if bpi_state = read3 then
					reply_tdata(15 downto 0) <= bpi_d_in;
					reply_tvalid <= '1';
				else
					reply_tvalid <= '0';
				end if;
			
			end if;
		end if;
    end process;

	-- Turn the program voltage and chip select on (1) or off (0).
	-- Note: This control is a hardware build option that is only present if requested.
	bpi_vpp <= bpi_enable;
	bpi_fcs_b <= not(bpi_enable);    
    
    -- Tie the reply MSBs to zero.
    reply_tdata(31 downto 16) <= (others => '0');
    
 	-- Pop the current instruction or data off the command FIFO.
    command_tready <= '1' when bpi_state = pop else '0';
    
    -- Address values are registered, so tie the control active.
    bpi_adv_b <= '0';

end structural;
