--*******************************************************************************************************************************
--
-- Description: This module limits the time that an interrupt request can remain active.
--					
-- Rev History: 01-10-2018:	Removed asynchronous reset.
--				09-28-2012:	Inital release.
--
--*******************************************************************************************************************************

library ieee;
use ieee.std_logic_1164.all;
use IEEE.numeric_std.all;

use work.model_specific_package.all;

entity interrupt_timer is
port (
    -- Reset and clocks.
	clk 				: in std_logic;						-- Bus clock.
	
	-- Command and status signals.
	int_timeout			: in std_logic_vector(27 downto 0);	-- Maximum time to hold an interrupt request active.
	int_response		: out std_logic_vector(27 downto 0);-- Time taken to service the last interrupt or timeout value.

    -- Interrupt req/ack controls.
	int_request			: in std_logic;						-- Interrupt request from the application logic.
	int_ack				: in std_logic;						-- Interrupt acknowledge from the EZDMA core.
	int_req_throttle	: out std_logic						-- Throttled interrupts to the EZDMA core.
);
end interrupt_timer;

architecture structural of interrupt_timer is

	signal interrupt_timer		: unsigned(27 downto 0);	-- Current value of the interrupt counter that acts as a timer.
	signal int_request_edge		: std_logic;				-- Register to falling edge detect the interrupt request.
	
begin

	process(clk)
	begin
		if rising_edge(clk) then
        	-- Register the interrupt request so that it can be edge detected.
        	int_request_edge <= int_request;
        	
        	-- There are no outstanding interrupt requests from the application logic.
			if int_request = '0' then
        		-- Reset the timer.
				interrupt_timer <= (others => '0');
        		-- Disable interrupt requests to the PLDA EZDMA core.
				int_req_throttle <= '0';
				-- Save the timer value on falling edge of int_request so that response time can be reported.
				if int_request_edge = '1' then
					int_response <= std_logic_vector(interrupt_timer);
				end if;
			-- The timer is counting when there is an outstanding interrupt request from the application logic.
			elsif interrupt_timer /= unsigned(int_timeout) then
        		-- Increment the timer.
				interrupt_timer <= interrupt_timer + x"0000001";
        		-- Enable an interrupt request to the PLDA EZDMA core.
				int_req_throttle <= '1';
			-- The timer has expired and the original interrupt to the PLDA EZDMA core has been acknowledged.
			elsif int_ack = '1' then
        		-- Disable interrupt requests to the PLDA EZDMA core.
				int_req_throttle <= '0';
			end if;
			
        end if;
    end process;

end structural;
